// 
// Copyright (C) 2015, NinjaTrader LLC <www.ninjatrader.com>.
// NinjaTrader reserves the right to modify or overwrite this NinjaScript component with each release.
//
#region Using declarations
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
using SharpDX;
using SharpDX.Direct2D1;
using System;

//added
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Xml.Serialization;
using NinjaTrader.Gui;

#endregion

namespace NinjaTrader.NinjaScript.ChartStyles
{
	/// <summary>
	/// Sim22 for NT8b9 Mar 2016.
	/// </summary>
	// TODO: Boolean for separate wick colors that changes config interface.
	public class OptimizedCandlestick : ChartStyle
	{
		private object icon;
		private SharpDX.Direct2D1.Brush _upBrush;
		private SharpDX.Direct2D1.Brush _downBrush;
		private SharpDX.Direct2D1.Brush _upOutBrush;
		private SharpDX.Direct2D1.Brush _dnOutBrush;
		private SharpDX.Direct2D1.Brush _dojiBrush;
		//private SharpDX.Direct2D1.Brush _dojiOutBrush;
		//private SharpDX.Direct2D1.Brush _dojiWickBrush;
		//private SharpDX.Direct2D1.Brush _upWickBrush;
		//private SharpDX.Direct2D1.Brush _downWickBrush;

		Vector2	_point0	= new Vector2();
		Vector2	_point1	= new Vector2();
		private SharpDX.RectangleF _rect = new SharpDX.RectangleF();
		private SharpDX.Direct2D1.StrokeStyle _styleWick;
		
		protected override void OnStateChange()
		{
			if (State == State.SetDefaults)
			{
				Name				= @"OptimizedCandlestick";
				Description			= "Plots customizable outline, wicks and bar opacity Candlesticks";
				ChartStyleType		= (ChartStyleType) 141;
				this.UpBrush		= System.Windows.Media.Brushes.Green;
				this.DownBrush 		= System.Windows.Media.Brushes.DarkRed;
				this.UpOutBrush 	= System.Windows.Media.Brushes.LimeGreen;
				this.DnOutBrush 	= System.Windows.Media.Brushes.Red;
				this.DojiBrush 		= System.Windows.Media.Brushes.Gray;
				WidthOutline		= 1f;
				WidthWick			= 1f;
				Opacity				= 85;
			}
			else if (State == State.Configure)
			{
				SetPropertyName("BarWidth",		Custom.Resource.NinjaScriptChartStyleBarWidth);
				SetPropertyName("DownBrush",	"Colors - Down bar");
				SetPropertyName("UpBrush",		"Colors - Up bar");
				
				Properties.Remove(Properties.Find("Stroke", true));
				Properties.Remove(Properties.Find("Stroke2", true));
			}
		}

		public override void OnRenderTargetChanged()
		{
			//Dispose of brushes.
			if (_upBrush != null) _upBrush.Dispose();
			if (_upOutBrush != null) _upOutBrush.Dispose();
			if (_downBrush != null) _downBrush.Dispose();
			if (_dnOutBrush != null) _dnOutBrush.Dispose();
			if (_dojiBrush != null) _dojiBrush.Dispose();
			if (_styleWick != null)	_styleWick.Dispose();
		
			if (RenderTarget != null)
			{
				//Set up lifetime objects
				_upBrush = UpBrush.ToDxBrush(RenderTarget);
				_upOutBrush = UpOutBrush.ToDxBrush(RenderTarget);
				_downBrush = DownBrush.ToDxBrush(RenderTarget);
				_dnOutBrush = DnOutBrush.ToDxBrush(RenderTarget);
				_dojiBrush = DojiBrush.ToDxBrush(RenderTarget);
				
				SharpDX.Direct2D1.StrokeStyleProperties ssPropsWick = new SharpDX.Direct2D1.StrokeStyleProperties();
				ssPropsWick.DashStyle								= DashStyleWick;
				_styleWick 			= new SharpDX.Direct2D1.StrokeStyle(RenderTarget.Factory, ssPropsWick);
			}
		}

		public override void OnRender(ChartControl chartControl, ChartScale chartScale, ChartBars chartBars)
		{
			Bars			bars			= chartBars.Bars;
			float			barWidth		= GetBarPaintWidth(BarWidthUI);

			SharpDX.Direct2D1.AntialiasMode oldAntialiasMode	= RenderTarget.AntialiasMode;
			RenderTarget.AntialiasMode							= SharpDX.Direct2D1.AntialiasMode.Aliased;
			
			for (int idx = chartBars.FromIndex; idx <= chartBars.ToIndex; idx++)
			{
				DrawBar(chartControl, chartScale, chartBars, idx, bars, barWidth);
			}
			
			RenderTarget.AntialiasMode = oldAntialiasMode;
		}


		public void DrawBar(ChartControl chartControl, ChartScale chartScale, ChartBars chartBars, int idx, Bars bars, float barWidth)
		{		
			// TODO: Are these sharpDX brushes??
			Brush		overriddenBarBrush		= chartControl.GetBarOverrideBrush(chartBars, idx);
			Brush		overriddenOutlineBrush	= chartControl.GetCandleOutlineOverrideBrush(chartBars, idx);
			
			
			double		closeValue				= bars.GetClose(idx);
			float		closeY					= chartScale.GetYByValue(closeValue);
			float		highY					= chartScale.GetYByValue(bars.GetHigh(idx));
			float		lowY					= chartScale.GetYByValue(bars.GetLow(idx));
			double		openValue				= bars.GetOpen(idx);
			float		openY					= chartScale.GetYByValue(openValue);
			float		x						= chartControl.GetXByBarIndex(chartBars, idx);

			Brush brush = _downBrush;
			Brush outBrush = _dnOutBrush;
		
			if (closeValue > openValue)
			{
				brush = _upBrush;
				outBrush = _upOutBrush;
			}
			else if (closeValue == openValue)
			{
				brush = _dojiBrush;
				outBrush = _dojiBrush;
			}
			
			//if bar is a doji
			if (Math.Abs(openY - closeY) < 0.0000001)
			{
				// Doji Line 
				_point0.X = x - barWidth * 0.5f;
				_point0.Y = closeY;
				_point1.X = x + barWidth * 0.5f;
				_point1.Y = closeY;

				TransformBrush(overriddenOutlineBrush ?? _dojiBrush, new RectangleF(_point0.X, _point0.Y - WidthOutline, barWidth, WidthOutline));
				RenderTarget.DrawLine(_point0, _point1, overriddenOutlineBrush ?? _dojiBrush, WidthOutline);
			}
			else
			{
				
				// Candle
				brush.Opacity	= Opacity * 0.01f;
				_rect.X		= x - barWidth * 0.5f + 0.5f;
				_rect.Y		= Math.Min(closeY, openY);
				_rect.Width	= barWidth - 1;
				_rect.Height	= Math.Max(openY, closeY) - Math.Min(closeY, openY);
				TransformBrush(overriddenBarBrush ?? brush, _rect);
				TransformBrush(overriddenOutlineBrush ?? outBrush, _rect);
				RenderTarget.FillRectangle(_rect, overriddenBarBrush ?? brush);
				RenderTarget.DrawRectangle(_rect, overriddenOutlineBrush ?? outBrush, WidthOutline);
			}

			// High wick
			if (highY < Math.Min(openY, closeY))
			{
				_point0.X = x;
				_point0.Y = highY;
				_point1.X = x;
				_point1.Y = Math.Min(openY, closeY);
				TransformBrush(overriddenOutlineBrush ?? outBrush, _rect);
				RenderTarget.DrawLine(_point0, _point1, overriddenOutlineBrush ?? outBrush, WidthWick, _styleWick);
			}

			// Low wick
			if (lowY > Math.Max(openY, closeY))
			{
				_point0.X = x;
				_point0.Y = lowY;
				_point1.X = x;
				_point1.Y = Math.Max(openY, closeY);
				TransformBrush(overriddenOutlineBrush ?? outBrush, _rect);
				RenderTarget.DrawLine(_point0, _point1, overriddenOutlineBrush ?? outBrush, WidthWick, _styleWick);
			}
		}
		
		public override int GetBarPaintWidth(int barWidth)
		{
			return 1 + 2 * (barWidth - 1) + 2 * (int) WidthOutline;
		}

		public override object Icon
		{
			get
			{
				if (icon == null)
				{
					icon = NinjaTrader.Gui.Tools.Icons.ChartChartStyle;
				}
				return icon;
			}
		}
		
		#region Properties

		[XmlIgnore]
		[Display(Name="Colors - Down bar outline", Description="Outline color of down bar", Order=1, GroupName="Chart Style")]
		public System.Windows.Media.Brush DnOutBrush
		{ get; set; }

		[Browsable(false)]
		public string DnOutBrushSerializable
		{
			get { return Serialize.BrushToString(DnOutBrush); }
			set { DnOutBrush = Serialize.StringToBrush(value); }
		}
		
		[XmlIgnore]
		[Display(Name="Colors - Down bar wick", Description="Wick color of down bar", Order=2, GroupName="Chart Style")]
		public System.Windows.Media.Brush DownWickBrush
		{ get; set; }

		[Browsable(false)]
		public string DownWickBrushSerializable
		{
			get { return Serialize.BrushToString(DownWickBrush); }
			set { DownWickBrush = Serialize.StringToBrush(value); }
		}
		
		[XmlIgnore]
		[Display(Name="Colors - Up bar outline", Description="Outline color of up bar", Order=2, GroupName="Chart Style")]
		public System.Windows.Media.Brush UpOutBrush
		{ get; set; }

		[Browsable(false)]
		public string UpOutBrushSerializable
		{
			get { return Serialize.BrushToString(UpOutBrush); }
			set { UpOutBrush = Serialize.StringToBrush(value); }
		}
	
		[XmlIgnore]
		[Display(Name="Colors - Up bar wick", Description="Wick color of up bar", Order=2, GroupName="Chart Style")]
		public System.Windows.Media.Brush UpWickBrush
		{ get; set; }

		[Browsable(false)]
		public string UpWickBrushSerializable
		{
			get { return Serialize.BrushToString(UpWickBrush); }
			set { UpWickBrush = Serialize.StringToBrush(value); }
		}
		
		[XmlIgnore]
		[Display(Name="Colors - Doji outline", Description="Color of doji bar", Order=3, GroupName="Chart Style")]
		public System.Windows.Media.Brush DojiBrush
		{ get; set; }

		[Browsable(false)]
		public string DojiBrushSerializable
		{
			get { return Serialize.BrushToString(DojiBrush); }
			set { DojiBrush = Serialize.StringToBrush(value); }
		}
		
		[Range(0, 100),NinjaScriptProperty]
		[Display(Name="Opacity % (0-100)", Description="Opacity of bar.", Order=4, GroupName="Chart Style")]
		public int Opacity
		{ get; set; }
		
		[Range(1.0f, 20.0f),NinjaScriptProperty]
		[Display(Name="Outline width", Description="Outline width.", Order=6, GroupName="Chart Style")]
		public float WidthOutline
		{ get; set; }
		
		[NinjaScriptProperty]
		[Display(Name="Wick dashstyle", Description="Wick dashstyle.", Order=7, GroupName="Chart Style")]
		public SharpDX.Direct2D1.DashStyle DashStyleWick
		{ get; set; }
		
		[Range(1.0f, 20.0f),NinjaScriptProperty]
		[Display(Name="Wick width", Description="Wick width.", Order=8, GroupName="Chart Style")]
		public float WidthWick
		{ get; set; }
		
		#endregion
	}
}
